pub struct DynSizedStructure<H: Header> { /* private fields */ }Expand description
A C ABI-compatible dynamically sized type with a common sized Header
and a dynamic amount of bytes without hidden implicit padding.
This structure combines a Header with the data described by that header
according to Header::total_size. Instances guarantee that the memory
requirements promised in the crate description are respected.
This can be a Multiboot2 header tag, information tag, boot information, or
a Multiboot2 header. It is the base for same-size casts to these
corresponding structures using DynSizedStructure::cast. Depending on the
context, the Header is different (basic header, boot information header,
header tag header, or boot information tag header).
ABI
This type uses the C ABI. The fixed Header portion is always there.
Further, there is a variable amount of payload bytes. Thus, this type can
only exist on the heap or references to it can be made by cast via fat
pointers. The main constructor is DynSizedStructure::ref_from_bytes.
As terminating padding might be necessary for the proper Rust type layout,
size_of_val(&self) might report additional padding bytes that are not
reflected by the actual payload. These additional padding bytes however
will be reflected in corresponding BytesRef instances from that this
structure was created.
Implementations
Sourceimpl<H: Header> DynSizedStructure<H>
impl<H: Header> DynSizedStructure<H>
Sourcepub fn ref_from_bytes(bytes: BytesRef<'_, H>) -> Result<&Self, MemoryError>
pub fn ref_from_bytes(bytes: BytesRef<'_, H>) -> Result<&Self, MemoryError>
Creates a new fat-pointer backed reference to a DynSizedStructure
from the given BytesRef.
Sourcepub fn ref_from_slice(bytes: &[u8]) -> Result<&Self, MemoryError>
pub fn ref_from_slice(bytes: &[u8]) -> Result<&Self, MemoryError>
Creates a new fat-pointer backed reference to a DynSizedStructure
from the given &[u8].
Sourcepub unsafe fn ref_from_ptr<'a>(ptr: NonNull<H>) -> Result<&'a Self, MemoryError>
pub unsafe fn ref_from_ptr<'a>(ptr: NonNull<H>) -> Result<&'a Self, MemoryError>
Creates a new fat-pointer backed reference to a DynSizedStructure
from the given thin pointer to the Header. It reads the total size
from the header.
Safety
The caller must ensure that ptr is readable for at least the size of
Header, and, once its reported total size is known, for that whole
range.
Sourcepub fn cast<T: MaybeDynSized<Header = H> + ?Sized>(&self) -> &T
pub fn cast<T: MaybeDynSized<Header = H> + ?Sized>(&self) -> &T
Performs a memory-safe same-size cast from the base-structure to a
specific MaybeDynSized. The idea here is to cast the generic
mostly semantic-free version to a specific type with fields that have
a clear semantic.
The provided T may be sized or dynamically sized. The source and
target have the same actual payload size and size_of_val.
Panics
Panics if T cannot represent the same allocation size. This should not
happen when all types follow their documented requirements.